@@ -19,6 +19,19 @@ def parse(inputs):
    return d
 
 
+def getValidatedInputs(inputs):
+   validatedInputs = {}
+   try:
+      params = parse(inputs)
+   except ValueError as e:
+      print(e)
+   else:
+      for param in params:
+         validatedInputs[param] = params[param].value
+
+   return validatedInputs
+
+
 def _get_extra_files(nbPath):
    """ Internal function to search the notebook for a cell tagged
    'FILES' with content 'EXTRA_FILES=xxx' where 'xxx' is a list of files
@@ -434,19 +447,37 @@ def _find_simTool(simToolName,simToolRevision=None):
             raise FileNotFoundError('No simtool named "%s, "' % (simToolName))
 
 
-def getNotebookInputs(nb):
-   incell = None
+def _getNotebookeCellYAMLcontent(nb,
+                                 yamlTag):
+   yamlDict = None
+# ignore lines up to and including %%yaml (cell magic)
+   yamlContent = None
+   yamlLineNumber = -1
    for cell in nb.cells:
-      if cell['source'].startswith('%%yaml INPUTS'):
-         incell = cell['source']
-         break
-   if incell is None:
-      return None
-   # remove first line (cell magic)
-   incell = incell.split('\n', 1)[1]
-   input_dict = yaml.load(incell, Loader=yaml.FullLoader)
+      cellSourceLines = cell['source'].split('\n')
+      lineNumber = 0
+      for cellSourceLine in cellSourceLines:
+         if cellSourceLine.startswith("%%%%yaml %s" % (yamlTag)):
+            yamlLineNumber = lineNumber
+            break
+         lineNumber += 1
 
-   return parse(input_dict)
+      if yamlLineNumber >= 0:
+         yamlContent = '\n'.join(cellSourceLines[yamlLineNumber+1:])
+         break
+      
+   if yamlContent:
+      yamlDict = yaml.load(yamlContent, Loader=yaml.FullLoader)
+
+   return yamlDict
+
+
+def getNotebookInputs(nb):
+   yamlDict = _getNotebookeCellYAMLcontent(nb,"INPUTS")
+   if yamlDict:
+      return parse(yamlDict)
+   else:
+      return None
 
 
 def getSimToolInputs(simToolLocation):
@@ -525,18 +556,11 @@ def _get_inputFiles(inputs):
 
 
 def getNotebookOutputs(nb):
-   incell = None
-   for cell in nb.cells:
-      if cell['source'].startswith('%%yaml OUTPUTS'):
-         incell = cell['source']
-         break
-   if incell is None:
+   yamlDict = _getNotebookeCellYAMLcontent(nb,"OUTPUTS")
+   if yamlDict:
+      return parse(yamlDict)
+   else:
       return None
-   # remove first line (cell magic)
-   incell = incell.split('\n', 1)[1]
-   out_dict = yaml.load(incell, Loader=yaml.FullLoader)
-
-   return parse(out_dict)
 
 
 def getSimToolOutputs(simToolLocation):
